#!/usr/bin/python

from datetime import timedelta, datetime

from dateutil.relativedelta import relativedelta
import fabric.colors as fc

from bosun.environ import fmt


DATE_SLICES = {
    'm': slice(4, 6),
    'd': slice(6, 8),
    'y': slice(0, 4),
    'h': slice(6, 8)
}

JOB_STATES = {
    'B': 'Array job has at least one subjob running.',
    'E': 'Job is exiting after having run.',
    'F': 'Job is finished.',
    'H': 'Job is held.',
    'M': 'Job was moved to another server.',
    'Q': 'Job is queued.',
    'R': 'Job is running.',
    'S': 'Job is suspended.',
    'T': 'Job is being moved to new location.',
    'U': 'Cycle-harvesting job is suspended due to keyboard activity.',
    'W': 'Job is waiting for its submitter-assigned start time to be reached.',
    'X': 'Subjob has completed execution or has been deleted.'
}


def genrange(*args):
    ''' Replacement for the range() builtin, accepting any argument that can be
        compared (like datetimes and timedeltas).'''

    if len(args) != 3:
        return range(*args)
    start, stop, step = args
    if start < stop:
        cmpe = lambda a, b: a < b
        inc = lambda a: a + step
    else:
        cmpe = lambda a, b: a > b
        inc = lambda a: a - step
    output = []
    while cmpe(start, stop):
        output.append(start)
        start = inc(start)
    return output


def total_seconds(tdelta):
    ''' Returns total seconds, given a timedelta '''
    return (tdelta.microseconds +
            (tdelta.seconds + tdelta.days * 24 * 3600) * 10 ** 6) / 10 ** 6


def calc_ETA(remh, remm, percent):
    ''' Calculate estimated remaining time '''
    diff = remh * 60 + remm
    minutes = diff / (percent or 1)
    remain = timedelta(minutes=minutes) - timedelta(minutes=diff)
    return remain.days / 24 + remain.seconds / 3600, (remain.seconds / 60) % 60


def print_ETA(environ, status, current):
    if environ['mode'] == 'warm':
        start = environ['restart']
    else:
        start = environ['start']
    begin = datetime.strptime(str(start), "%Y%m%d%H")
    end = (datetime.strptime(
        str(environ['finish']), "%Y%m%d%H") + relativedelta(days=+1))
    count = current - begin
    total = end - begin
    percent = (float(total_seconds(count)) / total_seconds(total))

    runh, runm = map(float, status['Time'].split(':'))
    remh, remm = calc_ETA(runh, runm, percent)
    print(fc.yellow('Model running time: %s, %.2f %% '
                    'completed, Estimated %02d:%02d'
                    % (status['Time'], 100 * percent, remh, remm)))


def hsm_full_path(environ):
    start = str(environ['start'])
    d = DATE_SLICES
    path = 'ic%02s/ic%04s/%02s' % (start[d['m']], start[d['y']], start[d['d']])

    if environ['type'] == 'atmos':
        cname = 'AGCM'
    elif environ['type'] == 'mom4p1_falsecoupled':
        cname = 'OGCM'
    elif environ['type'] == 'coupled':
        cname = 'CGCM'
    else:
        cname = 'GENERIC'

    full_path = fmt('{hsm}/{name}/dataout/%s' % path, environ)
    return full_path, cname


def clear_output(output):
    ''' It is very stupid to put echo in .bash_profile... '''
    patterns = ('HOME=', 'SUBMIT_HOME=', 'WORK_HOME=', 'TRANSFER_HOME=')
    return "\n".join(line for line in output.splitlines()
                     if not any(pattern in line for pattern in patterns))



#!/usr/bin/env python

import fabric.colors as fc
from fabric.decorators import task
from fabric.api import run as frun

from bosun import coupled, agcm, mom4
from bosun import tasks
from bosun.environ import env_options, fmt


@task
@env_options
def deploy(environ, **kwargs):
    '''Deploy cycle: prepare, compile.

    Depends on:
      prepare
      compilation
    '''
    print(fc.green("Started"))
    prepare(environ)
    compilation(environ)


@task
@env_options
def deploy_and_run(environ, **kwargs):
    '''Full model cycle: prepare, compile and run.

    Depends on:
      prepare
      compilation
      run
    '''
    print(fc.green("Started"))
    prepare(environ)
    compilation(environ)
    run(environ)


@task
@env_options
def compilation(environ, **kwargs):
    '''Compile code for model run and post-processing.

    Depends on:
      instrument_code
      compile_model
      check_code
    '''
    if environ['instrument']:
        tasks.check_code(environ)
        tasks.instrument_code(environ)
    elif tasks.check_code(environ):
        tasks.compile_model(environ)


@task
@env_options
def prepare(environ, **kwargs):
    '''Create all directories and put files in the right places.

    Depends on:
      prepare_expdir
      link_agcm_inputs
      prepare_workdir
    '''
    print(fc.yellow('Preparing expdir'))
    frun(fmt('mkdir -p {expdir}', environ))
    frun(fmt('mkdir -p {execdir}', environ))

    environ['model'].prepare(environ)
    frun(fmt('rsync -rtL --progress {expfiles}/exp/{name}/* {expdir}', environ))


@task
@env_options
def run(environ, **kwargs):
    '''Run the model.

    Depends on:
      run_model
    '''
    tasks.run_model(environ)


@task
@env_options
def archive(environ, **kwargs):
    '''Archive the outputs.

    Depends on:
      archive_model
    '''
    tasks.archive_model(environ)


@task
@env_options
def restart(environ, **kwargs):
    '''Restart the model.

    Depends on:
      run_model
      prepare_restart
    '''
    environ['mode'] = 'warm'
    tasks.run_model(environ)


@task
@env_options
def generate_grid(environ, **kwargs):
    tasks.prepare_expdir(environ)
    tasks.check_code(environ)
    mom4.compile_pre(environ)
    mom4.generate_grid(environ)


@task
@env_options
def make_xgrids(environ, **kwargs):
    tasks.prepare_expdir(environ)
    tasks.check_code(environ)
    mom4.compile_pre(environ)
    mom4.make_xgrids(environ)


@task
@env_options
def regrid_3d(environ, **kwargs):
    tasks.prepare_expdir(environ)
    tasks.check_code(environ)
    mom4.compile_pre(environ)
    mom4.regrid_3d(environ)


@task
@env_options
def regrid_2d(environ, **kwargs):
    tasks.prepare_expdir(environ)
    tasks.check_code(environ)
    mom4.compile_pre(environ)
    mom4.regrid_2d(environ)



#!/usr/bin/env python

import functools
from copy import deepcopy
from StringIO import StringIO

from fabric.api import run, get, prefix, hide
from fabric.contrib.files import exists
import rec_env


API_VERSION = 'v1'


class NoEnvironmentSetException(Exception):
    pass


class APIVersionException(Exception):
    pass


def env_options(func):
    '''Decorator that loads a YAML configuration file and expands
    cross-references.

    An example:
      workdir = ${HOME}/teste
      expdir = ${workdir}/exp
    would expand into:
      workdir = ${HOME}/teste
      expdir = ${HOME}/teste/exp
    The idea is: if a variable is not available to be replaced leave it there
    and the shell env should provide it.

    If an environ is passed for the original function it is used, if not the
    wrapper search for keyword argument 'config_file' and read the
    config file. Next step is to replace all the cross-referenced vars and then
    call the original function.
    '''

    def _wrapped_env(*args, **kw):
        if args:
            environ = args[0]
        else:
            try:
                exp_repo = kw.get('exp_repo', '${ARCHIVE_OCEAN}/exp_repos')
                name = kw['name']
            except KeyError:
                raise NoEnvironmentSetException

            environ = {'exp_repo': exp_repo,
                       'name': name,
                       'expfiles': '${HOME}/.bosun_exps'}
            environ = rec_env.expand_config_vars(environ)
            with hide('running', 'stdout', 'stderr', 'warnings'):
                if exists(fmt('{expfiles}', environ)):
                    run(fmt('rm -rf {expfiles}', environ))
                run(fmt('hg clone {exp_repo} {expfiles}', environ))
#                else:
#                    with cd(fmt('{expfiles}', environ)):
#                        run('hg pull -u ')

                temp_exp = StringIO()
                get(fmt('{expfiles}/exp/{name}/namelist.yaml', environ),
                    temp_exp)

            kw['expfiles'] = environ['expfiles']
            environ = load_configuration(temp_exp.getvalue(), kw)
            kw.pop('expfiles', None)
            kw.pop('name', None)
            kw.pop('exp_repo', None)
            temp_exp.close()

        return func(environ, **kw)

    return functools.update_wrapper(_wrapped_env, func)


def load_configuration(yaml_string, kw=None):
    environ = rec_env.load_configuration(yaml_string, kw)
    environ = update_model_type(environ)

    #if environ.get('API', 0) != API_VERSION:
    #    print fc.red('Error: Configuration outdated')
    #    ref = yaml.safe_load(
    #      os.path.join(os.path.dirname(__file__), 'api.yaml'))
    #    report_differences(environ, ref)
    #    raise APIVersionException

    if 'ensemble' in environ:
        environs = []
        ensemble = environ['ensemble']
        environ.pop('ensemble')
        for member in ensemble.keys():
            new_env = update_environ(environ, ensemble, member)
            new_env = update_model_type(environ)
            environs.append(new_env)

        # TODO: ignoring for now, but need to think on how to run
        # multiple environs (one for each ensemble member).
        # Maybe use the Fabric JobQueue, which abstracts
        # multiprocessing?

    return environ


def update_model_type(environ):
    from bosun import agcm, mom4, coupled

    mtype = environ.get('type', None)
    if mtype == 'coupled':
        environ['model'] = coupled
    elif mtype == 'mom4p1_falsecoupled':
        environ['model'] = mom4
    elif mtype == 'atmos':
        environ['model'] = agcm
    else:
        environ['model'] = None

    return environ


def update_component(new_env, nml, comp):
    ''' Update component in new_env. Needed because component has nested
        definitions, and a dict.update would not recurse. '''
    nml_vars = nml.get('vars', False)
    if nml_vars:
        if new_env[comp].get('vars', False):
            for k in nml_vars.keys():
                if new_env[comp]['vars'].get(k, None):
                    new_env[comp]['vars'][k].update(nml_vars[k])
                else:
                    new_env[comp]['vars'][k] = nml_vars[k]
        else:
            new_env[comp]['vars'] = nml_vars
        nml.pop('vars')

    new_env[comp].update(nml)
    return new_env


def update_environ(environ, ensemble, member):
    ''' Updates an existing environ '''
    new_env = deepcopy(environ)

    ocean_nml = ensemble[member].get('ocean_namelist', False)
    if ocean_nml:
        new_env = update_component(new_env, ocean_nml, 'ocean_namelist')
        ensemble[member].pop('ocean_namelist')

    atmos_nml = ensemble[member].get('agcm_namelist', False)
    if atmos_nml:
        new_env = update_component(new_env, atmos_nml, 'agcm_namelist')
        ensemble[member].pop('agcm_namelist')

    new_env['name'] = member
    new_env.update(ensemble[member])

    return new_env


def shell_env(environ, keys=None):
    '''Context manager that can send shell env variables to remote side.

    Really dumb context manager for Fabric, in fact. It just generates a new
    prefix with an export before the actual command. Something like

    $ export workdir=${HOME}/teste expdir=${HOME}/teste/exp && <cmd>
    '''

    if not keys:
        keys = environ.keys()
    valid = [k for k in keys
             if not isinstance(environ.get(k, None), (dict, list, tuple))]

    env_vars = " ".join(["=".join((key, str(value)))
                         for (key, value) in environ.items()
                         if key in valid])
    return prefix("export %s" % env_vars)


def fmt(string, environ):
    '''String formatting and sanitization function'''
    return string.format(**environ)


def report_differences(environ, ref):
    env_keys = set(environ.keys())

    req_keys = ref[API_VERSION]['Required']

    for key in req_keys:
        if not isinstance(key, dict):
            if key not in env_keys:
                print 'missing attribute: %s' % key
        else:
            for att, values in key.items():
                for v in values:
                    try:
                        environ[att].keys()
                    except AttributeError:
                        print 'missing attribute: %s[%s]' % (att, v)
                    else:
                        if v not in environ[att].keys():
                            print 'missing attribute: %s[%s]' % (att, v)



#!/usr/bin/env python

import sys
from StringIO import StringIO
import re
from datetime import datetime

from fabric.api import run, cd, get, put, prefix, settings, hide
from fabric.contrib.files import exists
from fabric.decorators import task
import fabric.colors as fc
from mom_utils import layout, nml_decode, yaml2nml

from bosun.environ import env_options, fmt, shell_env
from bosun.utils import print_ETA, JOB_STATES, hsm_full_path, clear_output


@task
@env_options
def prepare(environ, **kwargs):
    prepare_expdir(environ)
    prepare_workdir(environ)


@task
@env_options
def prepare_workdir(environ, **kwargs):
    '''Prepare output dir

    Used vars:
      workdir
      workdir_template
    '''
    print(fc.yellow('Preparing workdir'))
    run(fmt('mkdir -p {workdir}', environ))
    run(fmt('rsync -rtL --progress {workdir_template}/* {workdir}', environ))
    run(fmt('touch {workdir}/time_stamp.restart', environ))
    # TODO: lots of things.
    #  1) generate atmos inputs (gdas_to_atmos from oper scripts)
    #  2) copy restart files from somewhere (emanuel's spinup, for example)


@task
@env_options
def prepare_expdir(environ, **kwargs):
    run(fmt('mkdir -p {comb_exe}', environ))
    if environ.get('gengrid_run_this_module', False):
        run(fmt('mkdir -p {execdir}/gengrid', environ))
        run(fmt('mkdir -p {gengrid_workdir}', environ))
    if environ.get('make_xgrids_run_this_module', False):
        run(fmt('mkdir -p {execdir}/make_xgrids', environ))
        run(fmt('mkdir -p {make_xgrids_workdir}', environ))
    if environ.get('regrid_3d_run_this_module', False):
        run(fmt('mkdir -p {execdir}/regrid_3d', environ))
        run(fmt('mkdir -p {regrid_3d_workdir}', environ))
    if environ.get('regrid_2d_run_this_module', False):
        run(fmt('mkdir -p {execdir}/regrid_2d', environ))
        run(fmt('mkdir -p {regrid_2d_workdir}', environ))
    # Need to check if input.nml->ocean_drifters_nml->use_this_module is True
    run(fmt('mkdir -p {workdir}/DRIFTERS', environ))


@task
@env_options
def prepare_namelist(environ, **kwargs):
    ''' Read ocean namelist and update variables from environ as needed

    Used vars:
      name
      ocean_namelist
      npes
      dt_ocean
      dt_atmos
      dt_cpld
      days
      workdir

    Depends on:
      None
    '''
    input_file = StringIO()
    get(fmt('{ocean_namelist[file]}', environ), input_file)
    data = nml_decode(input_file.getvalue())
    input_file.close()
    output = StringIO()

    try:
        tkeys = set(
            environ['ocean_namelist']['vars'].keys()) & set(data.keys())
    except KeyError:
        pass
    else:
        for k in tkeys:
            keys = (set(environ['ocean_namelist']['vars'][k].keys())
                    & set(data[k].keys()))
            data[k].update([(ke, environ['ocean_namelist']['vars'][k][ke])
                            for ke in keys])

    if data['coupler_nml'].get('concurrent', False):
        data['ocean_model_nml']['layout'] = ("%d,%d"
                                             % layout(data['coupler_nml']['ocean_npes']))
    else:
        data['ocean_model_nml']['layout'] = ("%d,%d"
                                             % layout(int(environ['npes'])))

    data['ocean_model_nml']['dt_ocean'] = environ['dt_ocean']
    data['coupler_nml']['dt_atmos'] = environ['dt_atmos']
    data['coupler_nml']['dt_cpld'] = environ['dt_cpld']

    if 'days' in data['coupler_nml']:
        data['coupler_nml'].pop('days')
    if 'months' in data['coupler_nml']:
        data['coupler_nml'].pop('months')

    if ('days' in environ) & ('months' not in environ):
        data['coupler_nml']['days'] = environ['days']
    elif ('days' not in environ) & ('months' in environ):
        data['coupler_nml']['months'] = environ['months']
    else:
        print "Error, one should use days or months, not both or none"

    if environ['mode'] == 'warm':
        start = datetime.strptime(str(environ['restart']), "%Y%m%d%H")
    else:
        start = datetime.strptime(str(environ['start']), "%Y%m%d%H")
    data['coupler_nml']['current_date'] = start.strftime(
        "%Y, %m, %d, %H, 0, 0")

    if 'ocean_drifters_nml' in data.keys():
        if data['ocean_drifters_nml']['use_this_module']:
            environ['run_drifters_pos'] = True

    output.write(yaml2nml(data))

    put(output, fmt('{workdir}/input.nml', environ))
    output.close()


@task
@env_options
def compile_model(environ, **kwargs):
    keys = ['comp', 'code_dir', 'root', 'type', 'mkmf_template', 'executable']
    with shell_env(environ, keys=keys):
        with prefix(fmt('source {envconf}', environ)):
            with cd(fmt('{execdir}', environ)):
                run(fmt('/usr/bin/tcsh {ocean_makeconf}', environ))


@task
@env_options
def compile_post(environ, **kwargs):
    with shell_env(environ, keys=['root', 'platform']):
        with prefix(fmt('source {envconf}', environ)):
            with cd(environ['comb_exe']):
                run(fmt('make -f {comb_src}/Make_combine', environ))
    run(fmt('cp {root}/MOM4p1/src/shared/drifters/drifters_combine {comb_exe}/', environ))


@task
@env_options
def compile_pre(environ, **kwargs):
    with prefix(fmt('source {envconf}', environ)):
        if environ.get('gengrid_run_this_module', False):
            with shell_env(environ, keys=['root', 'platform', 'mkmf_template', 'executable_gengrid']):
                with cd(fmt('{execdir}/gengrid', environ)):
                    run(fmt('/usr/bin/tcsh {gengrid_makeconf}', environ))
        if environ.get('regrid_3d_run_this_module', False):
            with shell_env(environ, keys=['root', 'mkmf_template', 'executable_regrid_3d']):
                with cd(fmt('{execdir}/regrid_3d', environ)):
                    run(fmt('/usr/bin/tcsh {regrid_3d_makeconf}', environ))
        if environ.get('regrid_2d_run_this_module', False):
            with shell_env(environ, keys=['root', 'mkmf_template', 'executable_regrid_2d']):
                with cd(fmt('{execdir}/regrid_2d', environ)):
                    run(fmt('/usr/bin/tcsh {regrid_2d_makeconf}', environ))
    if environ.get('make_xgrids_run_this_module', False):
        with prefix(fmt('source {make_xgrids_envconf}', environ)):
            #run(fmt('cc -g -V -O -o {executable_make_xgrids} {make_xgrids_src} -I $NETCDF_DIR/include -L $NETCDF_DIR/lib -lnetcdf -lm -Duse_LARGEFILE -Duse_netCDF -DLARGE_FILE -Duse_libMPI', environ))
            fix_MAXLOCAL_make_xgrids(environ)
            run(fmt('cc -g -V -O -o {executable_make_xgrids} {make_xgrids_src} -I $NETCDF_DIR/include -L $NETCDF_DIR/lib -lnetcdf -lm -Duse_LARGEFILE -Duse_netCDF -DLARGE_FILE', environ))


def fix_MAXLOCAL_make_xgrids(environ):
    run(fmt("sed -i.bak -r -e 's/^#define MAXLOCAL.*$/#define MAXLOCAL 1e8/g' {make_xgrids_src}", environ))


@task
@env_options
def generate_grid(environ, **kwargs):
    run(fmt('cp {topog_file} {gengrid_workdir}/topog_file.nc', environ))
    with shell_env(
        environ, keys=['mom4_pre_npes', 'mom4_pre_walltime', 'RUNTM',
                       'executable_gengrid', 'gengrid_workdir',
                       'account', 'topog_file', 'platform']):
        with prefix(fmt('source {envconf}', environ)):
            with cd(fmt('{expdir}/runscripts/mom4_pre', environ)):
                out = run(fmt('/usr/bin/tcsh ocean_grid_run.csh', environ))


@task
@env_options
def regrid_3d(environ, **kwargs):
    run(fmt(
        'cp {regrid_3d_src_file} {regrid_3d_workdir}/src_file.nc', environ))
    with shell_env(environ, keys=['mom4_pre_npes', 'mom4_pre_walltime',
                                  'executable_regrid_3d', 'regrid_3d_workdir',
                                  'regrid_3d_dest_grid', 'regrid_3d_output_filename',
                                  'account', 'platform']):
        with prefix(fmt('source {envconf}', environ)):
            with cd(fmt('{expdir}/runscripts/mom4_pre', environ)):
                out = run(fmt('/usr/bin/tcsh regrid_3d_run.csh', environ))


@task
@env_options
def regrid_2d(environ, **kwargs):
    regrid_2d_prepare(environ)
    with shell_env(environ, keys=['mom4_pre_npes', 'mom4_pre_walltime',
                                  'executable_regrid_2d', 'regrid_2d_workdir',
                                  'regrid_2d_src_file', 'account', 'platform']):
        with prefix(fmt('source {envconf}', environ)):
            with cd(fmt('{expdir}/runscripts/mom4_pre', environ)):
                out = run(fmt('/usr/bin/tcsh regrid_2d_run.csh', environ))


@task
@env_options
def regrid_2d_prepare(environ, **kwargs):
    input_file = StringIO()
    get(fmt('{regrid_2d_namelist[file]}', environ), input_file)
    data = nml_decode(input_file.getvalue())
    input_file.close()
    output = StringIO()

    try:
        tkeys = set(
            environ['regrid_2d_namelist']['vars'].keys()) & set(data.keys())
    except KeyError:
        pass
    else:
        for k in tkeys:
            keys = (set(environ['regrid_2d_namelist']['vars'][k].keys())
                    & set(data[k].keys()))
            data[k].update([(ke, environ['regrid_2d_namelist']['vars'][k][ke])
                            for ke in keys])

    src_file = data['regrid_2d_nml']['src_file']
    run(fmt('cp %s {regrid_2d_workdir}/src_file.nc' % src_file, environ))
    data['regrid_2d_nml']['src_file'] = 'src_file.nc'
    output.write(yaml2nml(data))

    put(output, fmt('{regrid_2d_workdir}/input.nml', environ))
    output.close()


@task
@env_options
def make_xgrids(environ, **kwargs):
    with prefix(fmt('source {envconf}', environ)):
        with cd(fmt('{workdir}/gengrid', environ)):
            with settings(warn_only=True):
                out = run(fmt('{executable_make_xgrids} -o ocean_grid.nc -a {atmos_gridx},{atmos_gridy}', environ))
            # TODO: need to check why it returns 41 even if program ended right.
            # An appropriate return code is missing in make_xgrids.c ...
            if out.return_code == 41:
                run(fmt('cp ocean_grid.nc grid_spec_UNION.nc', environ))
                if exists(fmt('{workdir}/gengrid/ocean_grid?.nc', environ)):
                    run(fmt('for file in ocean_grid?.nc; do ncks -A $file grid_spec_UNION.nc; done', environ))
                run(fmt('ncks -A grid_spec.nc grid_spec_UNION.nc', environ))
                if exists(fmt('{workdir}/gengrid/grid_spec?.nc', environ)):
                    run(fmt('for file in grid_spec?.nc; do ncks -A $file grid_spec_UNION.nc; done', environ))
            else:
                sys.exit(1)


def get_coupler_dates(environ):
    res_time = run(fmt('tail -1 {workdir}/INPUT/coupler.res', environ))
    date_comp = [i for i in res_time.split('\n')[-1].split(' ') if i][:4]
    res_date = int("".join(
        date_comp[0:1] + ["%02d" % int(i) for i in date_comp[1:4]]))

    if 'cold' in environ['mode']:
        cmp_date = int(environ['start'])
    else:
        cmp_date = int(environ['restart'])

    return res_date, cmp_date


@task
@env_options
def check_restart(environ, **kwargs):
    prepare_restart(environ)
    if exists(fmt('{workdir}/INPUT/coupler.res', environ)):
        res_date, cmp_date = get_coupler_dates(environ)

        if res_date != cmp_date:
            print(fc.red('ERROR'))
            sys.exit(1)
    else:
        # TODO: check if it starts from zero (ocean forced)
        sys.exit(1)


@task
@env_options
def run_model(environ, **kwargs):
    ''' Submits ocean model

    Used vars:
      workdir
      platform
      walltime
      datatable
      diagtable
      fieldtable
      executable
      execdir
      mppnccombine
      comb_exe
      envconf
      expdir
      mode
      start
      restart
      finish
      npes
      name
      JobID_model

    Depends on:
      None
    '''
    print(fc.yellow('Submitting ocean model'))

    # Here goes a series of tests and preparations moved out from the
    #   mom4p1_coupled_run.csh, that are better be done here.
    # For some reason, this line bellow is not working. The dir does exist
    #   and this gives the error message, and do not stop here with the return.
    # Didn't understand.

    #if not exists(fmt('{workdir}/INPUT', environ)):
    #    print(fc.yellow(fmt("Missing the {workdir}/INPUT directory!", environ)))
    #    return
    #if not exists(fmt('{workdir}', environ)):
    #    print(fc.yellow(fmt("Missing the {workdir} directory!", environ)))
    #    run(fmt('mkdir -p {workdir}', environ))
    #if not exists(fmt('{workdir}/RESTART', environ)):
    #    print(fc.yellow(fmt("Missing the {workdir}/INPUT directory!", environ)))
    #    run(fmt('mkdir -p {workdir}/RESTART', environ))
    #if not exists(fmt('{workdir}/INPUT/grid_spec.nc', environ)):
    #    print(fc.yellow(fmt("ERROR: required input file does not exist {workdir}/INPUT/grid_spec.nc", environ)))
    #    return
    #if not exists(fmt('{workdir}/INPUT/ocean_temp_salt.res.nc', environ)):
    #    print(fc.yellow(fmt("ERROR: required input file does not exist {workdir}/INPUT/ocean_temp_salt.res.nc", environ)))
    #    return
    #run(fmt('cp {ocean_namelist} {workdir}/input.nml', environ))
    #run(fmt('cp {datatable} {workdir}/data_table', environ))
    #run(fmt('cp {diagtable} {workdir}/diag_table', environ))
    #run(fmt('cp {fieldtable} {workdir}/field_table', environ))

    keys = ['workdir', 'platform', 'walltime', 'datatable', 'diagtable',
            'fieldtable', 'executable', 'mppnccombine', 'comb_exe',
            'account']
    with shell_env(environ, keys=keys):
        with prefix(fmt('source {envconf}', environ)):
            with cd(fmt('{expdir}/runscripts', environ)):
                if environ.get('run_drifters_pos', False):
                    run(fmt('. set_pos_drifters.cray', environ))
                output = run(fmt('. run_g4c_model.cray {mode} {start} '
                                 '{restart} {finish} {npes} {name}', environ))
    environ['JobID_model'] = re.search(
        ".*JobIDmodel:\s*(.*)\s*", output).groups()[0]


@task
@env_options
def run_post(environ, **kwargs):
    ''' Submits ocean post-processing

    Used vars:
      JobID_model
      expdir
      workdir
      platform

    Depends on:
      None
    '''
    print(fc.yellow('Submitting ocean post-processing'))
    opts = ''
    if environ['JobID_model']:
        opts = '-W depend=afterok:{JobID_model}'
    with cd(fmt('{expdir}/runscripts', environ)):
        out = run(fmt(
            'qsub %s {workdir}/set_g4c_pos_m4g4.{platform}' % opts, environ))
        environ['JobID_pos_ocean'] = out.split('\n')[-1]

        if environ.get('run_drifters_pos', False):
            out = run(fmt('qsub %s {workdir}/run_pos_drifters.{platform}' %
                      opts, environ))
            environ['JobID_pos_ocean'] = out.split('\n')[-1]


@task
@env_options
def clean_experiment(environ, **kwargs):
    run(fmt('rm -rf {comb_exe}', environ))


def check_status(environ, status):
    if status['ID'] in environ.get('JobID_pos_ocean', ""):
        print(fc.yellow('Ocean post-processing: %s' % JOB_STATES[status['S']]))
    elif status['ID'] in environ.get('JobID_model', ""):
        fmsfile = fmt("{workdir}/fms.out", environ)
        if status['S'] == 'R':
            if not exists(fmsfile):
                print(fc.yellow('Preparing!'))
            else:
                with hide('running', 'stdout'):
                    line = run('tac %s | grep -m1 yyyy' % fmsfile).splitlines()[-1]
                current = re.search('(\d{4})/(\s*\d{1,2})/(\s*\d{1,2})\s(\s*'
                                    '\d{1,2}):(\s*\d{1,2}):(\s*\d{1,2})', line)
                if current:
                    current = datetime(*[int(i) for i in current.groups()])
                    print_ETA(environ, status, current)
                else:
                    print(fc.yellow('Preparing!'))
        else:
            print(fc.yellow('Model: %s' % JOB_STATES[status['S']]))


@task
@env_options
def archive(environ, **kwargs):
    full_path, cname = hsm_full_path(environ)

    run('mkdir -p %s/ocean/%s' % (full_path, cname))
    # TODO: copy OGCM output ({workdir}/dataout)

    run('mkdir -p %s/output' % full_path)
    with cd(fmt('{workdir}', environ)):
        with settings(warn_only=True):
            out = run(fmt('ls -1 *fms.out *logfile.*.out input.nml '
                      'set_g4c_model*out.txt *_table *diag_integral.out '
                      'set_g4c_pos_m4g4*out.txt *time_stamp.out 2> /dev/null', environ))
        files = clear_output(out).splitlines()
        for f in files:
            if exists(f):
                if f not in ('data_table', 'diag_table', 'field_table', 'input.nml'):
                    run(fmt('gzip %s' % f, environ))
                    f = f + '.gz'
                run(fmt('mv %s %s/output/' % (f, full_path), environ))

    run('mkdir -p %s/restart' % full_path)
    with cd(fmt('{workdir}/RESTART', environ)):
        # TODO: check date in coupler.res!
        run(fmt('tar czvf {finish}.tar.gz coupler* ice* land* ocean*', environ))
        run(fmt('mv {finish}.tar.gz %s/restart/' % full_path, environ))
    with cd(fmt('{workdir}', environ)):
        run(fmt('tar czvf INPUT.tar.gz INPUT/ --exclude="*.res*"', environ))
        run(fmt('mv INPUT.tar.gz %s/restart/' % full_path, environ))


@task
@env_options
def prepare_restart(environ, **kwargs):
    '''Prepare restart for new run'''

    # TODO: check if it starts from zero (ocean forced)

    if 'cold' in environ['mode']:
        cmp_date = str(environ['start'])
    else:
        cmp_date = str(environ['restart'])

    with settings(warn_only=True):
        with cd(fmt('{workdir}/INPUT', environ)):
            full_path, cname = hsm_full_path(environ)
            run(fmt('tar xf %s/restart/%s.tar.gz' % (full_path, cmp_date), environ))


@task
@env_options
def verify_run(environ, **kwargs):
    if 'cold' in environ['mode']:
        cmp_date = str(environ['start'])
    else:
        cmp_date = str(environ['restart'])
    run(fmt('grep "Total runtime" {workdir}/%s.fms.out' % cmp_date[:8], environ))
    # TODO: need to check post processing!



 #!/usr/bin/env python

from __future__ import with_statement
from __future__ import print_function
import time
from datetime import datetime

from fabric.api import run, cd, prefix, settings, hide
import fabric.colors as fc
from fabric.contrib.files import exists
from fabric.decorators import task
from dateutil.relativedelta import relativedelta

from bosun.environ import env_options, fmt
from bosun.utils import genrange


__all__ = ['check_code', 'check_status', 'clean_experiment', 'compile_model',
           'instrument_code', 'kill_experiment', 'run_model', 'archive_model']


@task
@env_options
def instrument_code(environ, **kwargs):
    '''Instrument code using Cray's perftools.

    Used vars:
      executable
      expdir

    Depends on:
      compile_model
    '''
    print(fc.yellow('Rebuilding executable with instrumentation'))
    with prefix('module load perftools'):
        compile_model(environ)
        if exists(fmt('{executable}+apa', environ)):
            run(fmt('rm {executable}+apa', environ))
        run(fmt('pat_build -O {expdir}/instrument_coupler.apa '
                '-o {executable}+apa {executable}', environ))
        environ['executable'] = fmt('{executable}+apa', environ)


@task
@env_options
def run_model(environ, **kwargs):
    '''Run the model

    Used vars:
      expdir
      mode
      start
      restart
      restart_interval
      finish
      name
      days
      type

    Depends on:
      agcm.prepare_namelist
      mom4.prepare_namelist
      agcm.run_model
      mom4.run_model
      coupled.run_model
      agcm.run_post
      mom4.run_post
      check_status
      prepare_restart
    '''
    print(fc.yellow('Running model'))

    begin = datetime.strptime(str(environ['restart']), "%Y%m%d%H")
    end = datetime.strptime(str(environ['finish']), "%Y%m%d%H")
    if environ['restart_interval'] is None:
        delta = relativedelta(end, begin)
    else:
        interval, units = environ['restart_interval'].split()
        if not units.endswith('s'):
            units = units + 's'
        delta = relativedelta(**dict([[units, int(interval)]]))

    for period in genrange(begin, end, delta):
        finish = period + delta
        if finish > end:
            finish = end

        if environ['mode'] == 'cold':
            environ['restart'] = finish.strftime("%Y%m%d%H")
            environ['finish'] = environ['restart']
        else:
            environ['restart'] = period.strftime("%Y%m%d%H")
            environ['finish'] = finish.strftime("%Y%m%d%H")

        environ['model'].check_restart(environ)

        # TODO: set months and days? only use days?
        if period.day == finish.day:
            environ['months'] = (12 * (finish.year - period.year) +
                                 finish.month - period.month)
        else:
            environ['days'] = (finish - period).days

        # TODO: set restart_interval in input.nml to be equal to delta

        environ['model'].prepare_namelist(environ)
        environ['model'].run_model(environ)
        environ['model'].run_post(environ)

        while check_status(environ, oneshot=True):
            time.sleep(environ.get('status_sleep_time', 60))

        environ['model'].verify_run(environ)
        environ['model'].archive(environ)
        environ['mode'] = 'warm'


def _get_status(environ):
    ''' Retrieve PBS status from remote side '''
    with settings(warn_only=True):
        with hide('running', 'stdout'):
            job_ids = " ".join([environ[k] for k in environ.keys()
                                if "JobID" in k])
            data = run(fmt("qstat -a %s" % job_ids, environ))
    statuses = {}
    if data.succeeded:
        header = None
        for line in data.split('\n'):
            if header:
                info = line.split()
                try:
                    statuses[info[0]] = dict(zip(header.split()[1:], info))
                except IndexError:
                    pass
            elif line.startswith('Job ID'):
                header = line
    return statuses


@task
@env_options
def check_status(environ, **kwargs):
    print(fc.yellow('Checking status'))

    statuses = _get_status(environ)
    if not statuses:
        print(fc.yellow('No jobs running.'))
        return False

    while statuses:
        for status in statuses.values():
            environ['model'].check_status(environ, status)

        if kwargs.get('oneshot', False):
            statuses = {}
        else:
            time.sleep(environ.get('status_sleep_time', 60))
            statuses = _get_status(environ)
        print()

    return True


@task
@env_options
def kill_experiment(environ, **kwargs):
    print(fc.yellow('Killing experiment'))
    statuses = _get_status(environ)
    if statuses:
        for status in statuses.values():
            s = status['Jobname']
            if (s in fmt('M_{name}', environ) or
                s in fmt('C_{name}', environ) or
                    s in fmt('P_{name}', environ)):
                run('qdel %s' % status['ID'].split('-')[0])


@task
@env_options
def clean_experiment(environ, **kwargs):
    run(fmt('rm -rf {expdir}', environ))
    run(fmt('rm -rf {rootexp}', environ))
    run(fmt('rm -rf {execdir}', environ))
    run(fmt('rm -rf {workdir}', environ))
    environ['model'].clean_experiment(environ)


@task
@env_options
def compile_model(environ, **kwargs):
    '''Compile model and post-processing tools.

    Used vars:
      envconf
      execdir
      makeconf
      comb_exe
      comb_src
      envconf_pos
      posgrib_src
      PATH2
    '''
    print(fc.yellow("Compiling code"))
    environ['model'].compile_model(environ)
    environ['model'].compile_pre(environ)
    environ['model'].compile_post(environ)


@task
@env_options
def check_code(environ, **kwargs):
    ''' Checks out code in the remote side.

    Used vars:
      clean_checkout
      code_dir
      code_repo
      code_dir
      code_branch
      revision
      executable
    '''
    print(fc.yellow("Checking code"))

    changed = False
    if environ['clean_checkout']:
        run(fmt('rm -rf {code_dir}', environ))
        changed = True

    if not exists(environ['code_dir']):
        print(fc.yellow("Creating new repository"))
        run(fmt('mkdir -p {code_dir}', environ))
        run(fmt('hg clone {code_repo} {code_dir}', environ))
        changed = True

    with cd(environ['code_dir']):
        print(fc.yellow("Updating existing repository"))

        # First check if there is any change in repository, or
        # if requesting a different branch/revision
        with settings(warn_only=True):
            res = run(fmt('hg incoming -b {code_branch}', environ))
        if res.return_code == 0:  # New changes!
            run('hg pull')
            changed = True
        rev = environ.get('revision', None)
        curr_rev = run('hg id -i').strip('+')
        run(fmt('hg update {code_branch} --clean', environ))
        if rev and rev != 'last' and rev != curr_rev:
            run(fmt('hg update -r{revision}', environ))
            changed = True

        # Need to check if executables exists!
        if not exists(environ['executable']):
            changed = True

    return changed


@task
@env_options
def archive_model(environ, **kwargs):
    environ['model'].archive(environ)



#!/usr/bin/env python

from __future__ import with_statement
from __future__ import print_function
import re

from fabric.api import run, cd, prefix
from fabric.decorators import task
import fabric.colors as fc

from bosun import mom4, agcm
from bosun.environ import env_options, fmt, shell_env
from bosun.utils import JOB_STATES


__all__ = ['compile_model', 'run_model', 'prepare', 'compile_pre', 'compile_post', 'archive']


@task
@env_options
def run_model(environ, **kwargs):
    ''' Submits coupled model

    Used vars:
      workdir
      platform
      walltime
      datatable
      diagtable
      fieldtable
      executable
      execdir
      TRC
      LV
      rootexp
      mppnccombine
      comb_exe
      envconf
      expdir
      mode
      start
      restart
      finish
      npes
      name
      JobID_model

    Depends on:
      None
    '''
    print(fc.yellow('Submitting coupled model'))

    trunc = "%04d" % environ['TRC']
    lev = "%03d" % environ['LV']
    env_vars = environ.copy()
    env_vars.update({'TRUNC': trunc, 'LEV': lev})

    keys = ['workdir', 'platform', 'walltime', 'datatable', 'diagtable',
            'fieldtable', 'executable', 'execdir', 'TRUNC', 'LEV', 'LV',
            'rootexp', 'mppnccombine', 'comb_exe', 'account', 'DHEXT']
    with shell_env(env_vars, keys=keys):
        with prefix(fmt('source {envconf}', environ)):
            with cd(fmt('{expdir}/runscripts', environ)):
                output = run(fmt('. run_g4c_model.cray {mode} {start} '
                                 '{restart} {finish} {npes} {name}', environ))
    environ['JobID_model'] = re.search(".*JobIDmodel:\s*(.*)\s*", output).groups()[0]


@task
@env_options
def compile_model(environ, **kwargs):
    keys = ['comp', 'code_dir', 'root', 'type', 'mkmf_template', 'executable']
    with shell_env(environ, keys=keys):
        with prefix(fmt('source {envconf}', environ)):
            with cd(fmt('{execdir}', environ)):
                run(fmt('/usr/bin/tcsh -e {cpld_makeconf}', environ))


@task
@env_options
def prepare(environ, **kwargs):
    mom4.prepare_expdir(environ)
    mom4.prepare_workdir(environ)
    agcm.link_agcm_inputs(environ)


@task
@env_options
def compile_pre(environ, **kwargs):
    mom4.compile_pre(environ)
    agcm.compile_pre(environ)


@task
@env_options
def compile_post(environ, **kwargs):
    mom4.compile_post(environ)
    agcm.compile_post(environ)


@task
@env_options
def run_post(environ, **kwargs):
    mom4.run_post(environ)
    agcm.run_post(environ)


@task
@env_options
def prepare_namelist(environ, **kwargs):
    mom4.prepare_namelist(environ)
    agcm.prepare_namelist(environ)


@task
@env_options
def check_restart(environ, **kwargs):
    mom4.check_restart(environ)
    agcm.check_restart(environ)


@task
@env_options
def clean_experiment(environ, **kwargs):
    mom4.clean_experiment(environ)
    agcm.clean_experiment(environ)


def check_status(environ, status):
    mom4.check_status(environ, status)
    if status['ID'] in environ.get('JobID_pos_atmos', ""):
        print(fc.yellow('Atmos post-processing: %s' % JOB_STATES[status['S']]))


@task
@env_options
def archive(environ, **kwargs):
    mom4.archive(environ)
    agcm.archive(environ)


@task
@env_options
def prepare_restart(environ, **kwargs):
    mom4.prepare_restart(environ)
    agcm.prepare_restart(environ)


@task
@env_options
def verify_run(environ, **kwargs):
    mom4.verify_run(environ)
    agcm.verify_run(environ)



#!/usr/bin/env python

from __future__ import with_statement
from __future__ import print_function
from StringIO import StringIO
import re
from datetime import datetime

from fabric.api import run, cd, get, put, prefix, hide, settings
from fabric.decorators import task
import fabric.colors as fc
from fabric.contrib.files import exists
from mom_utils import nml_decode, yaml2nml

from bosun.environ import env_options, fmt, shell_env
from bosun.utils import total_seconds, JOB_STATES, print_ETA
from bosun.utils import hsm_full_path, clear_output


def format_atmos_date(date):
    ''' Format date as required by atmos model '''
    out = str(date)
    return "%s,%s,%s,%s" % (out[8:], out[6:8], out[4:6], out[0:4])


@task
@env_options
def prepare_namelist(environ, **kwargs):
    ''' Read atmos namelist and update variables from environ as needed

    Used vars:
      name
      atmos_namelist
      TRC
      LV
      dt_atmos
      start
      restart
      finish
      rootexp
      workdir

    Depends on:
      None
    '''
    input_file = StringIO()
    get(fmt('{agcm_namelist[file]}', environ), input_file)
    data = nml_decode(input_file.getvalue())
    input_file.close()
    output = StringIO()

    try:
        tkeys = set(
            environ['atmos_namelist']['vars'].keys()) & set(data.keys())
    except KeyError:
        pass
    else:
        for k in tkeys:
            keys = set(environ['atmos_namelist']['vars'][k]
                       .keys()) & set(data[k].keys())
            data[k].update([(
                ke, environ['atmos_namelist']['vars'][k][ke]) for ke in keys])

    trunc = "%04d" % environ['TRC']
    lev = "%03d" % environ['LV']

    data['MODEL_RES']['trunc'] = "%04d" % environ['TRC']
    data['MODEL_RES']['vert'] = environ['LV']
    data['MODEL_RES']['dt'] = environ['dt_atmos']
    data['MODEL_RES']['IDATEI'] = format_atmos_date(environ['start'])
    data['MODEL_RES']['IDATEW'] = format_atmos_date(environ['restart'])
    data['MODEL_RES']['IDATEF'] = format_atmos_date(environ['finish'])
    data['MODEL_RES']['DHEXT'] = environ.get('DHEXT', 0)
    if environ.get('DHEXT', 0) != 0:
        begin = datetime.strptime(environ['restart'], "%Y%m%d%H")
        end = datetime.strptime(environ['finish'], "%Y%m%d%H")
        nhext = total_seconds(end - begin) / 3600
    else:
        nhext = 0
    data['MODEL_RES']['NHEXT'] = nhext

    # TODO: is this environ['agcm_model_inputs'] ?
    data['MODEL_RES']['path_in'] = fmt(
        '{rootexp}/AGCM-1.0/model/datain', environ)

    data['MODEL_RES']['dirfNameOutput'] = (
        fmt('{workdir}/model/dataout/TQ%sL%s' % (trunc, lev), environ))

    output.write(yaml2nml(data,
                          key_order=['MODEL_RES', 'MODEL_IN', 'PHYSPROC',
                                     'PHYSCS', 'COMCON']))

    # HACK: sigh, this is needed to run atmos post processing, even if we
    # don't use these forecasts.
    output.write("""
 17
   6.0 12.0  18.0  24.0
  30.0 36.0  42.0  48.0
  54.0 60.0  66.0  72.0
  84.0 96.0 120.0 144.0
 168.0
""")

    put(output, fmt('{workdir}/MODELIN', environ))
    output.close()


@task
@env_options
def run_post(environ, **kwargs):
    ''' Submits atmos post-processing

    Used vars:
      JobID_model
      expdir
      workdir
      platform

    Depends on:
      None
    '''
    print(fc.yellow('Submitting atmos post-processing'))
    opts = ''
    if environ['JobID_model']:
        opts = '-W depend=afterok:{JobID_model}'
    with cd(fmt('{expdir}/runscripts', environ)):
        out = run(
            fmt('qsub %s {workdir}/set_g4c_posgrib.cray' % opts, environ))
        environ['JobID_pos_atmos'] = out.split('\n')[-1]


@task
@env_options
def run_model(environ, **kwargs):
    ''' Submits atmos model

    Used vars:
      rootexp
      workdir
      executable
      walltime
      execdir
      platform
      TRC
      LV
      envconf
      expdir
      start
      restart
      finish
      npes
      name
      JobID_model

    Depends on:
      None
    '''
    print(fc.yellow('Submitting atmos model'))

    trunc = "%04d" % environ['TRC']
    lev = "%03d" % environ['LV']
    env_vars = environ.copy()
    env_vars.update({'TRUNC': trunc, 'LEV': lev})

    keys = ['rootexp', 'workdir', 'TRUNC', 'LEV', 'executable', 'walltime',
            'execdir', 'platform', 'LV']
    with shell_env(env_vars, keys=keys):
        with prefix(fmt('source {envconf}', environ)):
            with cd(fmt('{expdir}/runscripts', environ)):
                output = run(fmt('. run_atmos_model.cray run {start} {restart} '
                                 '{finish} {npes} {name}', environ))
    job_id = re.search(".*JobIDmodel:\s*(.*)\s*", output).groups()[0]
    environ['JobID_model'] = job_id


@task
@env_options
def compile_pre(environ, **kwargs):
    with shell_env(environ, keys=['PATH2']):
        with prefix(fmt('source {envconf_pos}', environ)):
            with cd(fmt('{pre_atmos}/sources', environ)):
                fix_atmos_makefile()
                run(fmt('make cray', environ))


@task
@env_options
def compile_post(environ, **kwargs):
    with shell_env(environ, keys=['PATH2']):
        with prefix(fmt('source {envconf_pos}', environ)):
            with cd(environ['posgrib_src']):
                fix_atmos_makefile()
                run(fmt('make cray', environ))


@task
@env_options
def compile_model(environ, **kwargs):
    with shell_env(environ, keys=['root', 'executable']):
        with prefix(fmt('source {envconf}', environ)):
            with cd(fmt('{execdir}', environ)):
                run(fmt('make -f {atmos_makeconf}', environ))


@task
@env_options
def prepare(environ, **kwargs):
    prepare_expdir(environ)
    prepare_workdir(environ)


@task
@env_options
def prepare_workdir(environ, **kwargs):
    '''Prepare output dir

    Used vars:
      workdir
      workdir_template
    '''
    print(fc.yellow('Preparing workdir'))
    run(fmt('mkdir -p {workdir}', environ))
    run(fmt('rsync -rtL --progress {workdir_template}/* {workdir}', environ))
    run(fmt('touch {workdir}/time_stamp.restart', environ))
    # TODO: lots of things.
    #  1) generate atmos inputs (gdas_to_atmos from oper scripts)
    #  2) copy restart files from somewhere (emanuel's spinup, for example)


@task
@env_options
def prepare_expdir(environ, **kwargs):
    run(fmt('mkdir -p {PATH2}', environ))
    link_agcm_inputs(environ)


@task
@env_options
def check_restart(environ, **kwargs):
    if 'warm' in environ['mode']:
        prepare_restart(environ)
        run(fmt('ls {workdir}/model/dataout/TQ{TRC:04d}L{LV:03d}/*{start}{restart}F.unf*outatt*', environ))


@task
@env_options
def prepare_inputs(environ, **kwargs):
    #TODO: copy data to pre/datain (look at oper experiment)
    with cd(fmt('pre_atmos/scripts', environ)):
        fix_atmos_runpre(environ)
        envvars = {
            'dirhome': fmt('{pre_atmos}', environ),
            'dirdata': fmt('{rootexp}/AGCM-1.0', environ),
            'direxe': fmt('{execdir}', environ)
        }
        with shell_env(envvars):
            run(fmt('bash/runAll.bash', environ))


@task
@env_options
def link_agcm_inputs(environ, **kwargs):
    '''Copy AGCM inputs for model run and post-processing to the right place

    Used vars:
      rootexp
      agcm_pos_inputs
      agcm_model_inputs
    '''
    for comp in ['model', 'pos']:
        print(fc.yellow(fmt("Linking AGCM %s input data" % comp, environ)))
        if not exists(fmt('{rootexp}/AGCM-1.0/%s/datain' % comp, environ)):
            run(fmt('mkdir -p {rootexp}/AGCM-1.0/%s/datain' % comp, environ))
        run(fmt('cp -R {agcm_%s_inputs}/* '
                '{rootexp}/AGCM-1.0/%s/datain' % (comp, comp), environ))


def fix_atmos_makefile():
    '''Stupid pre and pos makefiles...'''
    run("sed -i.bak -r -e 's/^PATH2/#PATH2/g' Makefile")


def fix_atmos_runpre(environ):
    '''Stupid pre runscripts...'''
    run(fmt("sed -i.bak -r -e 's/^export DATA=.*$/export DATA={start}/g' bash/runAll.bash", environ))
    run(fmt("sed -i.bak -r -e 's|^export dirhome|#export dirhome|g' bash/runAll.bash", environ))
    run(fmt("sed -i.bak -r -e 's|^export dirdata|#export dirdata|g' bash/runAll.bash", environ))
#    for script in ls('bash/*.bash'):
#        run("sed -i.bak -r -e 's|^export direxe|#export direxe|g' %s" % script)
    #TODO: which parts of preprocessing to run? Comment all the vars in runAll,
    #and set as appropriate?


@task
@env_options
def clean_experiment(environ, **kwargs):
    run(fmt('rm -rf {PATH2}', environ))


def check_status(environ, status):
    if status['ID'] in environ.get('JobID_pos_atmos', ""):
        print(fc.yellow('Atmos post-processing: %s' % JOB_STATES[status['S']]))
    elif status['ID'] in environ.get('JobID_model', ""):
        if status['S'] == 'R':
            with hide('running', 'stdout'):
                fcts = run(fmt('find {workdir}/model/dataout/ -iname "*.fct.*" | sort', environ))
            date = re.search('.*(\d{10})F.*', fcts.splitlines()[-1])
            if date:
                current = datetime.strptime(date.groups()[0], "%Y%m%d%H")
                print_ETA(environ, status, current)
            else:
                print(fc.yellow('Preparing!'))
        else:
            print(fc.yellow('Model: %s' % JOB_STATES[status['S']]))


@task
@env_options
def archive(environ, **kwargs):
    full_path, cname = hsm_full_path(environ)

    run('mkdir -p %s/atmos/%s' % (full_path, cname))
    # TODO: copy AGCM output ({workdir}/pos/dataout)

    run('mkdir -p %s/output' % full_path)
    with cd(fmt('{workdir}', environ)):
        with settings(warn_only=True):
            out = run(fmt('ls -1 MODELIN Out.MPI.* set_post*out.txt '
                      'set_g4c_posgrib*out.txt POSTIN-GRIB '
                      'set_g4c_poseta*out.txt 2>/dev/null', environ))
        files = clear_output(out).splitlines()
        for f in files:
            if exists(f):
                if f not in ('MODELIN', 'POSTIN-GRIB'):
                    run(fmt('gzip %s' % f, environ))
                    f = f + '.gz'
                run(fmt('mv %s %s/output/' % (f, full_path), environ))

    run('mkdir -p %s/restart' % full_path)
    with cd(fmt('{workdir}/model/dataout/TQ{TRC:04}L{LV:03}', environ)):
        run(fmt('tar czvf GFCTNMC{start}{finish}F.unf.TQ{TRC:04}L{LV:03}.tar.gz '
            'GFCTNMC{start}{finish}F.unf.TQ{TRC:04}L{LV:03}.*P???', environ))
        run(fmt('mv GFCTNMC{start}{finish}F.unf.TQ{TRC:04}L{LV:03}.tar.gz '
            '%s/restart/' % full_path, environ))
        run(fmt('rm GFCTNMC{start}{finish}F.unf.TQ{TRC:04}L{LV:03}.*P???',
            environ))


@task
@env_options
def prepare_restart(environ, **kwargs):
    '''Prepare restart for new run'''
    with settings(warn_only=True):
        out = run(fmt('ls {workdir}/model/dataout/TQ{TRC:04d}L{LV:03d}/*{start}{restart}F.unf*outatt*', environ))
    if out.failed:
        with cd(fmt('{workdir}/model/dataout/TQ{TRC:04}L{LV:03}', environ)):
            full_path, cname = hsm_full_path(environ)
            run(fmt('tar xf %s/restart/GFCTNMC{start}{restart}F.unf.TQ{TRC:04}L{LV:03}.tar.gz' % full_path, environ))


@task
@env_options
def verify_run(environ, **kwargs):
    pass



# -*- coding: utf-8 -*-
try:
    from setuptools import setup, find_packages
except ImportError:
    import distribute_setup
    distribute_setup.use_setuptools()
    from setuptools import setup, find_packages

long_desc = ''' '''
requires = [
  'Fabric>=1.6.0',
  'PyYAML',
  'mom-utils',
  'python-dateutil',
  'rec_env'
]

setup(
    name='bosun',
    version='1.0.1',
    url='https://github.com/luizirber/bosun',
    download_url='https://github.com/luizirber/bosun',
    license='PSF',
    author='Luiz Irber, Guilherme Castelao',
    author_email='luiz.irber@gmail.com, castelao@gmail.com',
    description='',
    long_description=long_desc,
    zip_safe=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: Python Software Foundation License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Topic :: Scientific/Engineering',
    ],
    platforms='any',
    scripts=["bin/bosun"],
    packages=find_packages(),
    include_package_data=True,
    install_requires=requires,
)
